
// Enchanting Bridge.
//
// Aims to do:
//
// 1.  Launch Enchanting (by executing any command passed to it on the command line).
//     - terminate process when Enchanting quits
//     - if no command is provided, ignore step 1
// 2.  Establish communication with Enchanting (using the Scratch socket protocol).
// 3.  Periodically scan the USB bus for an NXT to connect to.  
//     - establish a connection if there is one and report to Enchanting.
//     - may need to check that firmware on the brick.
//     - if we lose our connection, inform Enchanting.
// 4.  Execute commands issued by Enchanting.
//     - build and upload NXJ programs.
//     - command the NXT to run or stop
// 5.  Relay commands between Enchanting and the NXT.
//     - specifically, broadcast commands and global variable updates

import java.io.*;


public class EnchantingBridge extends Thread {
	private static boolean verbose = true;

	public static void main(String args[]) {

		// Spawn the daemon threads
		new Dispatcher();
		
		// If we were told which program to run,
		if (args.length > 0) {
			// run until it stops.
			if (verbose)
				System.out.print("Running "); 
			SaferExec se = new SaferExec(SaferExec.NO_TIMEOUT);
			String result = se.exec(args);
			System.out.println(result);
			System.out.println("Enchanting Front End has quit; Back End is exiting."); 			
		} else {
			System.out.println("Press Control-C to quit."); 			
			// just run in an infinite loop
			while (true) { 
				try { 
					sleep(10000); 
				} catch (InterruptedException e) {
					throw new RuntimeException(e);
				}
			}
		}
	}
}
