CommandBlockMorph subclass: #ReporterBlockMorph
	instanceVariableNames: 'isBoolean '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-Blocks'!

!ReporterBlockMorph methodsFor: 'initialization' stamp: 'jm 3/31/2005 19:39'!
initialize

	super initialize.
	isBoolean _ false.
	self extent: 80@20.
! !


!ReporterBlockMorph methodsFor: 'drawing' stamp: 'jens 7/7/2010 00:17'!
drawBooleanOn: aCanvas 

	| right topY bottomY xInset c l t cArgs relevant |
	self computeHighlightColors.

	l _ OrderedCollection new.

	right _ self width.
	topY _ bottomY _ self height // 2.
	self height even ifTrue: [topY _ bottomY - 1].
	xInset _ 0.
	[topY >= 0] whileTrue: [
		topY = 0 ifTrue: [c _ highlightColor1].
		topY = 1 ifTrue: [c _ highlightColor2].
		topY > 1 ifTrue: [c _ color].
		self addHLineFrom: xInset to: right - xInset y: topY color: c to: l.
		(topY > 1) & (xInset > 0) ifTrue: [
			self addPoint: xInset@topY color: highlightColor1 to: l.
			self addPoint: (right - xInset - 1)@topY color: shadowColor to: l.].

		c _ bottomY = (self height - 1) ifTrue: [shadowColor] ifFalse: [color].
		self addHLineFrom: xInset to: right - xInset y: bottomY color: c to: l.
		xInset > 0 ifTrue: [
			self addPoint: xInset@bottomY color: shadowColor to: l.
			self addPoint: (right - xInset - 1)@bottomY color: shadowColor to: l].

		xInset _ xInset + 1.	
		bottomY _ bottomY + 1.
		topY _ topY - 1].

	cArgs _ self nonControlFlowSubmorphs select: [:s | 
		(s isKindOf: CArgSlotMorph) 
		or: [(s isKindOf: CReporterSlotMorph)
		or: [(s isKindOf: MultiArgMorph) 
			and: [#(loop reporterSlot) includes: s type]]]].

	cArgs isEmpty 
		ifFalse: [
			cArgs do: [:ca |
				relevant _ ca bounds.
				(ca isKindOf: MultiArgMorph) ifTrue: [
					relevant _ ca transparentRect].
				t _ OrderedCollection new.
				l do: [:pair |
					self addAreasOf: pair first outside: (relevant insetBy: 3) color: pair second to: t].
			l _ t]].
	
	l do: [:pair |
		aCanvas fillRectangle: pair first color: pair second ].
! !

!ReporterBlockMorph methodsFor: 'drawing' stamp: 'jens 7/7/2010 00:18'!
drawNumericOn: aCanvas 

	| right topY bottomY radius xInset c l t cArgs relevant |
	self computeHighlightColors.

	l _ OrderedCollection new.

	right _ self width.
	topY _ bottomY _ radius _ self height // 2.
	self height even ifTrue: [topY _ bottomY - 1].
	[topY >= 0] whileTrue: [
		xInset _ radius - (radius squared - (radius - topY - 1) squared) sqrt rounded.

		topY = 0 ifTrue: [c _ highlightColor1].
		topY = 1 ifTrue: [c _ highlightColor2].
		topY > 1 ifTrue: [c _ color].
		self addHLineFrom: xInset to: right - xInset y: topY color: c to: l.
		(xInset > 0) & (topY > 1) ifTrue: [
			self addPoint: xInset@topY color: highlightColor1 to: l.
			self addPoint: (right - xInset - 1)@topY color: highlightColor1 to: l].

		c _ bottomY = (self height - 1) ifTrue: [shadowColor] ifFalse: [color].
		self addHLineFrom: xInset to: right - xInset y: bottomY color: c to: l.
		xInset > 0 ifTrue: [
			self addPoint: xInset@bottomY color: shadowColor to: l.
			self addPoint: (right - xInset - 1)@bottomY color: shadowColor to: l].

		bottomY _ bottomY + 1.
		topY _ topY - 1].
	
	cArgs _ self nonControlFlowSubmorphs select: [:s | 
		(s isKindOf: CArgSlotMorph) 
		or: [(s isKindOf: CReporterSlotMorph)
		or: [(s isKindOf: MultiArgMorph) 
			and: [#(loop reporterSlot) includes: s type]]]].

	cArgs isEmpty 
		ifFalse: [
			cArgs do: [:ca |
				relevant _ ca bounds.
				(ca isKindOf: MultiArgMorph) ifTrue: [
					relevant _ ca transparentRect].
				t _ OrderedCollection new.
				l do: [:pair |
					self addAreasOf: pair first outside: (relevant insetBy: 3) color: pair second to: t].
			l _ t]].
	
	l do: [:pair |
		aCanvas fillRectangle: pair first color: pair second ].
! !

!ReporterBlockMorph methodsFor: 'drawing' stamp: 'jens 1/28/2010 23:21'!
drawOn: aCanvas 

	self contrastColor.

	self computeHighlightColors.
	isBoolean
		ifTrue: [self drawBooleanOn: aCanvas]
		ifFalse: [self drawNumericOn: aCanvas].

	self drawInlinePaletteOn: aCanvas
! !


!ReporterBlockMorph methodsFor: 'watcher' stamp: 'jm 11/30/2007 19:23'!
createWatcher
	"Return a new watcher for me."

	| newWatcher |
	newWatcher _ WatcherMorph new setCategoryColor: self color.
	self argumentCount = 1
		ifTrue: [newWatcher target: receiver selector: selector parameter: self args first]
		ifFalse: [newWatcher target: receiver selector: selector parameter: nil].

	^ newWatcher
! !

!ReporterBlockMorph methodsFor: 'watcher' stamp: 'jm 12/1/2007 20:49'!
getAssociatedSprite
	"Return my receiver."

	^ receiver
! !

!ReporterBlockMorph methodsFor: 'watcher' stamp: 'jm 12/1/2007 20:58'!
toggleWatcher
	"Toggle between hiding and showing a watcher for this block."

	| frame w palette |
	(frame _ self ownerThatIsA: ScratchFrameMorph) ifNil: [^ self].
	(w _ frame watcherForBlock: self)
		ifNil: [frame showWatcher: self createWatcher]
		ifNotNil: [w delete].
			
	palette _ self ownerThatIsA: ScratchBlockPaletteMorph.
	palette ifNotNil: [palette updateWatcherButtonsForFrame: frame].
! !


!ReporterBlockMorph methodsFor: 'object i/o' stamp: 'jm 5/15/2005 10:27'!
fieldsVersion

	^ 2
! !

!ReporterBlockMorph methodsFor: 'object i/o' stamp: 'jm 5/15/2005 10:27'!
initFieldsFrom: anObjStream version: classVersion

	super initFieldsFrom: anObjStream version: classVersion.
	classVersion = 1 ifTrue: [^ self].

	"fields added in version 2"
	self initFieldsNamed: #(
		isBoolean
	) from: anObjStream.

! !

!ReporterBlockMorph methodsFor: 'object i/o' stamp: 'jm 5/11/2005 10:50'!
storeFieldsOn: anObjStream

	super storeFieldsOn: anObjStream.
	self storeFieldsNamed: #(
		isBoolean
	) on: anObjStream.
! !


!ReporterBlockMorph methodsFor: 'other' stamp: 'jens 1/8/2010 17:39'!
acceptsTypeOf: aBlockMorph
	"Answer true if I can accept a dropped reporter of the given type. Answer false by default."

	aBlockMorph isReporter ifFalse: [^ false].
	self isBlockTemplate ifTrue: [^ false ].
	(owner isKindOf: CustomCommandBlockMorph) ifTrue: [
		owner isTemplate ifTrue: [^false]].
	^ aBlockMorph isBooleanReporter = self isBooleanReporter or: [
		aBlockMorph isAbstractFunctionCall ]
! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jm 3/30/2005 09:21'!
computeHighlightColors

	| h s v |
	h _ color hue.
	s _ color saturation.
	v _ color brightness.
	highlightColor1 _ Color h: h s: (s - 0.13) v: (v + 0.17).
	highlightColor2 _ Color h: h s: s v: (v + 0.06).
	shadowColor _ color mixed: 0.8 with: Color black.
! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'CWB 12/11/2010 19:28'!
fixBlockLayout
	"Update the positions of my submorphs."

	"(Deleting functions doesn't work well with this version control system.)"
	super fixBlockLayout! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jm 3/28/2005 12:59'!
isBoolean: aBoolean

	isBoolean _ aBoolean.
! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jm 3/28/2005 19:14'!
isBooleanReporter

	^ isBoolean
! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jm 3/31/2005 19:38'!
isReporter
	"Answer true if this block reports a value."

	^ true
! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jens 2/24/2011 12:23'!
showValue

	| gotError msg tooltip w worldBounds |

	(#(doReport doMakeNewList doCallWithArgs doPauseThreadReporter) includes: selector) ifTrue: [^self showLambdaValue ].
"
	(self allMorphs detect: [:m | (m isKindOf: ReporterBlockMorph) and: [m isSpecialForm]] ifNone: [nil])
		isNil ifFalse: [ receiver ifNotNil: [^ self showLambdaValue]].
"
	gotError _ false.
	[msg _ self evaluate ] ifError: [
		gotError _ true.
		msg _ 'Error!!'].

	(msg isKindOf: Morph)
		ifTrue: [(msg respondsTo: #thumbnailImageForm)
			ifTrue: [msg _ msg thumbnailImageForm]
			ifFalse: [msg _ msg imageForm ]]
		ifFalse: [msg _ msg asString ].

	(self isBooleanReporter or: [msg = 'Error!!']) ifTrue: [
		msg _ msg localized].
	tooltip _ (ScratchReporterToolTipMorph string: msg for: self)
		left: self right;
		bottom: self top.
	ScratchTranslator isRTL ifTrue: [tooltip right: self left].
	gotError ifTrue: [tooltip messageColor: Color red darker].

	w _ self world.
	w ifNil: [^ self].
	w addMorphFront: tooltip.
	((worldBounds _ w bounds) containsRect: tooltip bounds) ifFalse:
		[tooltip bounds: (tooltip bounds translatedToBeWithin: worldBounds)].

	self setProperty: #toolTip toValue: tooltip.
	tooltip initialCursorPoint: Sensor cursorPoint.
	tooltip startStepping.
! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jens 6/28/2010 20:40'!
tabFields

	| fields |
	fields _	OrderedCollection new.
	submorphs do: [:m |
		((m isKindOf: ReporterBlockMorph) or: [m isKindOf: MultiArgMorph]) 
			ifTrue: [fields addAll: m tabFields ]
			ifFalse: [(m isKindOf: ExpressionArgMorph)
				ifTrue: [fields add: m]]].
	^ fields! !

!ReporterBlockMorph methodsFor: 'other' stamp: 'jm 3/20/2009 19:54'!
toggleProcess

	| toolTip |
	toolTip _ self valueOfProperty: #toolTip.
	toolTip ifNotNil: [
		toolTip delete.
		self setProperty: #toolTip toValue: nil.
		^ self].

	self showValue.
! !


!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 7/20/2010 23:41'!
asLambda

	| lambda |
	lambda _ ReporterScriptBlockMorph new
		receiver: ScriptableScratchMorph new;
		isSpecialForm: false;
		minArgs: 1;
		commandSpec: '%s';
		selector: #function.
	lambda replaceArgMorph: (lambda argumentAt: 1) by: self fullCopy.
	^lambda evaluate! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 4/16/2010 23:27'!
bodyLeft

	^ self left + (self height // 2)
! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 4/16/2010 23:08'!
bodyLeft: x
	"move me"

	self left: self left + (x - self bodyLeft)
! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 4/16/2010 23:28'!
bodyRight

	^ self right - (self height // 2)
! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 4/16/2010 23:34'!
bodyRight: x
	"change my width, don't move"

	self width: self width + (x - self bodyRight)
! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 7/20/2010 23:17'!
declaredType

	(owner isKindOf: CommandBlockMorph) ifTrue: [
		^ owner typeOfArg: self ].
	^ nil! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 7/8/2010 23:33'!
fixBlockLayoutNoWrap
	"Update the positions of my submorphs."

	| mList maxH h x y argH |

	(#(function functionWithArgs doReport doCallWithArgs doCallBlockWithArgs) includes: selector) ifTrue: [
		self updateIsBoolean; layoutChanged].

	blockLayoutNeeded ifFalse: [^ self].
	cachedForm _ nil.
	cachedFeedbackForm _ nil.

	mList _ self nonControlFlowSubmorphs.
	maxH _ 0.
	mList do: [:m |
		argH _ nil.
		(m isKindOf: ArgMorph) ifTrue: [
			m fixArgLayout.
			argH _ m height].
		(m isKindOf: BlockMorph) ifTrue: [
			m fixBlockLayout.
			argH _ m height.
			m isBlockTemplate ifTrue: [
				argH _ argH + 4]].
			argH ifNil: [argH _ m height].
		
		maxH _ maxH max: argH].
	h _ (maxH + 4) max: 17.

	x _ isBoolean ifTrue: [8] ifFalse: [4].
	(mList size > 0 and: [mList first isKindOf: StringMorph]) ifTrue: [x _ x + 2].

	mList do: [:m |
		(m isKindOf: StringMorph) 
			ifTrue: [(m isKindOf: BlockLabelFragmentMorph) 
				ifFalse: [m color: self labelColor]].

		y _ (h - m height) // 2.
		m position: self position + (x@y).
		x _ x + m width + 3].
	x _ x + (isBoolean ifTrue: [1] ifFalse: [-3]).

	self extent: (x + 4) @ h.
	(self ownerThatIsA: ScratchBlockPaletteMorph) ifNotNil: [
		(self ownerThatIsA: ScratchBlockPaletteMorph) fixLayout].

! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'CWB 7/19/2011 07:12'!
fixBlockLayoutWrap
	"Update the positions of my submorphs."

	| parts lines line h x y minXForArgs w space left maxX maxHeight extraSpace mh |

	(#(function functionWithArgs doReport doCallWithArgs doCallBlockWithArgs) includes: selector) ifTrue: [
		self updateIsBoolean; layoutChanged].

	(self isCustomBlock and: [self isTemplate]) ifTrue: [^ self fixBlockLayoutNoWrap].
	blockLayoutNeeded ifFalse: [^ self].

	cachedForm _ nil.
	cachedFeedbackForm _ nil.

	parts _ self nonControlFlowSubmorphs.
	parts isEmpty ifTrue: [^self].
	lines _ OrderedCollection new.


	"distribute parts horizontally and cut up into lines"

	line _ OrderedCollection new.
	x _ self left + 5.
	left _ x.
	maxX _ x.
	minXForArgs _ self left.
	space _ 4.

	parts do: [:m |
	((m isKindOf: CArgSlotMorph) or: [(m isKindOf: CReporterSlotMorph) or: [(m isKindOf: MultiArgMorph) and: [m isWrapping]]])
		ifTrue: [ 
			line isEmpty ifFalse: [
				lines add: line].
			m left: self left - 5.
			lines add: (OrderedCollection with: m).
			(m isKindOf: MultiArgMorph)
				ifTrue: [m fixArgLayout]
				ifFalse: [m fixBlockLayout].
			((m isKindOf: CArgSlotMorph) or: [m isKindOf: CReporterSlotMorph])
				ifTrue: [m width: 40].
			line _ OrderedCollection new.
			maxX _ maxX max: (left + m width).
			x _ left ]
		ifFalse: [
			x - left > self wrapWidth ifTrue: [
				lines add: line.
				line _ OrderedCollection new.
				x _ left].
			(m isKindOf: BlockMorph) ifTrue: [m fixBlockLayout].
			(m isKindOf: ArgMorph) ifTrue: [m fixArgLayout].
			(m isKindOf: StringMorph)
				ifTrue: [m color: self labelColor]
				ifFalse: [x _ x max: minXForArgs].

			m left: x.
			(m right - left > 300 and: [line isEmpty not])
				ifTrue: [
					lines add: line.
					line _ OrderedCollection new.
					m left: left.
					line add: m.
					x _ left]
				ifFalse: [ line add: m].
				x _ x + m fullBounds width + space.
				maxX _ maxX max: x]].
	x = left ifFalse: [
		lines add: line].
	w _ maxX - self left max: 4.

	"distribute lines vertically"

	space _ 1.
	y _ self top "+ 2" "+ space".
	lines do: [:eachLine |
		maxHeight _ 14.
		eachLine do: [:m |
			((m isKindOf: ReporterBlockMorph)
			and: [m isBlockTemplate or: [ 
			((m isKindOf: CArgSlotMorph) 
			or: [m isKindOf: OSlotMorph]) and: [(m isKindOf: CReporterSlotMorph) not and: [m isEmptySlot not]]] ])
				ifTrue: [mh _ m height + 10]

				ifFalse: [ ((m isKindOf: CArgSlotMorph) or: [(m isKindOf: OSlotMorph) and: [(m isKindOf: CReporterSlotMorph) not]])
					ifTrue: [mh _ m height + 4]
					ifFalse: [mh _ m height + 2]].


			maxHeight _ mh max: maxHeight].
		eachLine do: [:m |
			m top: y + (maxHeight - m height // 2) + 1].
		y _ y + maxHeight + space].
	h _ y - self top + space.


	"adjust lines to round / hex shape"

	extraSpace _ 0.
	lines size > 1 
		ifTrue: [extraSpace _ h // 2]
		ifFalse: [ isBoolean
			ifTrue: [extraSpace _ h // 3]].

	parts do: [:m|
		m left: m left + extraSpace].

	w _ w + (extraSpace * 2).

	"adjust CArgSlots"

	parts do: [:m |
		((m isKindOf: CArgSlotMorph) or: [m isKindOf: CReporterSlotMorph])
			ifTrue: [m width: w - (extraSpace * 2) + 6]].

	((parts last isKindOf: CArgSlotMorph) or: [parts last isKindOf: CReporterSlotMorph]) ifTrue: [
		h _ h + space + 5].


	"stack nested Reporters more efficiently:"

	lines size > 1 ifTrue: [
		self height: h.
		lines do: [:l |
			| offset |
			((l first respondsTo: #bodyLeft) and: [(isBoolean and: [l first isBooleanReporter not]) not])
				ifTrue: [ offset _ l first bodyLeft - self bodyLeft]
				ifFalse: [ offset _ l first left - self bodyLeft].
				l do: [:m |
					self isBooleanReporter
						ifTrue: [m left: m left - offset + 2]
						ifFalse: [m left: m left - offset]]].

		maxX _ self left + 5.
		lines do: [:l |
			((l last respondsTo: #bodyRight)  and: [(isBoolean and: [l last isBooleanReporter not]) not])
				ifTrue: [maxX _ maxX max: l last bodyRight]
				ifFalse: [maxX _ maxX max: l last right]].
		self isBooleanReporter
			ifTrue: [self bodyRight: maxX + 2]
			ifFalse: [self bodyRight: maxX].

		"adjust Multi-CArgSlots"

		parts do: [:m | (m isKindOf: MultiArgMorph)
			ifTrue: [m loopsWidth: self bodyRight - self bodyLeft]]]

	ifFalse: [ self extent: w @ h].

	(self ownerThatIsA: ScratchBlockPaletteMorph) ifNotNil: [
		(self ownerThatIsA: ScratchBlockPaletteMorph) fixLayout].

! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 2/14/2011 21:16'!
isAbstractFunctionCall

	^ #(doCustomBlock doReport doCallWithArgs doCallBlockWithArgs doCallBlockWithArgList getLine:ofList: get:) includes: selector
! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 3/22/2010 23:10'!
lambdaMenu

	| menu choice |
	menu _ CustomMenu new.

	(owner isKindOf: ScratchBlockPaletteMorph) ifTrue: [^ self].

	(owner isKindOf: ReporterScriptBlockMorph)
		ifFalse: [ menu add: 'quote' action: #lambdafied ].

	(choice _ menu localize; startUp) ifNotNil: [
		self perform: choice ].
	World activeHand showTemporaryCursor: nil! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 3/22/2010 21:42'!
lambdafied

	| lambda |
	lambda _ (ReporterScriptBlockMorph new
		color: (ScriptableScratchMorph blockColorFor: 'operators');
		receiver: receiver;
		isSpecialForm: false;
		minArgs: 1;
		selector: #function;
		isAtomic: false;
		commandSpec: 'the %s block').

	(owner isKindOf: BlockMorph) ifTrue: [
		owner replaceArgMorph: self by: lambda.
		lambda replaceArgMorph: lambda argMorphs first by: self.
		^lambda].

	owner ifNil: [^lambda].

	lambda position: self position.
	owner addMorph: lambda.
	lambda replaceArgMorph: lambda argMorphs first by: self.
	^lambda
	
	
! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 3/22/2010 21:54'!
mouseDown: evt
	"Handle a mouse click. Left button either drags or performs click action. Right button brings up a menu."

	evt hand newKeyboardFocus: nil.
	evt shiftPressed ifTrue: [
		^ self lambdaMenu].
	super mouseDown: evt! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 7/28/2010 22:50'!
showLambdaValue

	| gotError msg tooltip w worldBounds zombie |
	gotError _ false.

	[self start.
	zombie _ scratchProc.
	scratchProc stepUntilDoneFor: (receiver ownerThatIsA: ScratchStageMorph).
	msg _ zombie answer ] ifError: [
		gotError _ true.
		msg _ 'Error!!'].

	(msg isKindOf: Morph)
		ifTrue: [(msg respondsTo: #displayForm)
			ifTrue: [msg _ msg displayForm ]
			ifFalse: [msg _ msg imageForm ]]
		ifFalse: [msg _ msg asString ].

	(self isBooleanReporter or: [msg = 'Error!!']) ifTrue: [
		msg _ msg localized].
	tooltip _ (ScratchReporterToolTipMorph string: msg for: self)
		left: self right;
		bottom: self top.
	ScratchTranslator isRTL ifTrue: [tooltip right: self left].
	gotError ifTrue: [tooltip messageColor: Color red darker].

	w _ self world.
	w addMorphFront: tooltip.
	((worldBounds _ w bounds) containsRect: tooltip bounds) ifFalse:
		[tooltip bounds: (tooltip bounds translatedToBeWithin: worldBounds)].

	self setProperty: #toolTip toValue: tooltip.
	tooltip initialCursorPoint: Sensor cursorPoint.
	tooltip startStepping.! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'jens 1/6/2010 22:14'!
updateIsBoolean

	| arg |
	argMorphs size < 1 ifTrue: [^ self ].
	arg _ argMorphs first.

	(arg isKindOf: ReporterBlockMorph) ifFalse: [
		isBoolean _ false. ^self].

	arg isBooleanReporter ifTrue: [isBoolean _ true. ^self ].

	isBoolean _ false

	! !

!ReporterBlockMorph methodsFor: 'byob lambdas' stamp: 'CWB 1/12/2013 05:27'!
wrapWidth
"If a block is wider than this many pixels, make it wrap"	

	^ 350.! !


!ReporterBlockMorph methodsFor: 'geometry' stamp: 'CWB 11/12/2012 11:26'!
paddedHeightWhenPlacedAt: place
"How heigh is this morph when placed as the nth element,
along a line of elements in a command block?"

	| h |

	h _ super paddedHeightWhenPlacedAt: place.

	self isBlockTemplate ifTrue: [
		(place = 1) ifTrue: [ ^ h + 10 ].
		^ h + 5.
	].
	^ h.
! !
