'From MIT Squeak 0.9.4 (June 1, 2003) [No updates present.] on 10 October 2011 at 10:35:45 am'!
Morph subclass: #ScratchFrameMorph
	instanceVariableNames: 'topPane viewerPane scriptsPane stageFrame workPane titlePane libraryPane menuPanel stageButtonsPanel readoutPane logoMorph projectTitleMorph flagButton fillScreenFlag paintingInProgress projectDirectory projectName projectInfo author loginName loginPassword watcherPositions shuffledCostumeNames justSaved viewModeButtons viewMode lastViewMode viewModeButtonsPanel toolbarPanel lastWeDoPoll pauseButton connectionInterfaces quitFlag enchantingDialog blockEditors '
	classVariableNames: 'AllowSharing Clipboard DefaultNotes DefaultSprite Fonts FontsXO IsXO ProcessedMetaCommands ScratchServers ScratchSkin ScratchSkinXO ShareServer ShareServerPath SupportServer SupportServerPath TakeOverScreen UseErrorCatcher Version VersionDate VisibleDrives WorkpaneExtent '
	poolDictionaries: ''
	category: 'Scratch-UI-Panes'!

!FileDirectory methodsFor: 'file operations' stamp: 'CWB 6/26/2010 20:45'!
fileInContents
	"Given a directory with files named like '1-Object.st', files in all .st files in numerical order"

	| filesToFileIn filesInOrder index |

	"Grab the names of all files that look like they can be filed in"
	filesToFileIn _ self fileNamesMatching: '*.st'.

	"File in classes based in the order corresponding to the number in front of the file name
	(so that base classes are filed in before subclasses)"

	filesInOrder _ filesToFileIn asSortedCollection sortBlock: [:a :b | a asNumber < b asNumber].

	index _ 1.
	'Importing source code ...' 
		displayProgressAt: Sensor cursorPoint 
		from: 1 to: filesInOrder size
		during: [:bar |
			filesInOrder do: [:name | 
				CrLfFileStream fileIn: (self fullNameFor: name). 
				bar value: index.
				index _ index + 1]].



! !

!FileDirectory methodsFor: 'file operations' stamp: 'CWB 7/2/2010 20:21'!
fileOutClassesChangedSince: aDate

	"Goes through all classes to find ones that have changed since a given date, and files those out within the directory.  Filenames are prepended with a number representing the depth in the class heirarchy, so that they can be filed in in the correct order by 'fileInContents'"

	| allClasses classesToExport reallyOldDate index |

	reallyOldDate _ '01/01/78' asDate.

	"Get a list of all classes (but not metaclasses)"
	allClasses _ Object selectSubclasses: [ :aClass | (aClass isMeta) not ].

	"Prune the list to only those that have changed since the given date"
	index _ 1.
	'Searching for modified classes...' 
		displayProgressAt: Sensor cursorPoint
		from: 1 to: allClasses size
		during: 
			[:bar |
				classesToExport _ allClasses 
					select: [ :aClass | 
						| latestChange |
						index _ index + 1.
						bar value: index.
						latestChange _ 
							((Utilities dateOfLatestChangeToClass: aClass) ifNil: [ reallyOldDate ])
							max:
							((Utilities dateOfLatestChangeToClass: aClass class) ifNil: [ reallyOldDate ]).
						latestChange > aDate ]].
	
	index _ 1.
	'Exporting modified classes...'
		displayProgressAt: Sensor cursorPoint
		from: 1 to: classesToExport size
		during: [:bar |
			classesToExport do: [ :aClass |
				"Files are named as follows: depth-class.st 
				(where depth is a number representing how deep 
				the class is in the class heirarchy"

				| depth filename filestream |

				depth _ aClass allSuperclasses size.
				filename _ depth asString, '-', aClass name , FileDirectory dot, 'st'.
				self deleteFileNamed: filename.		"Ensure we can overwrite the file"
				filestream _ CrLfFileStream defaultToCRLF newFileNamed: (self fullNameFor: filename).

				filestream header. "timeStamp."	"Omit the timestamp!!"
				aClass sharedPools size > 0 
					ifTrue: [self fileOutSharedPoolsOn: filestream].
		
				aClass fileOutOn: filestream moveSource: false toFile: 0.
				filestream trailer; close. ]]

			

		! !


!FileStream class methodsFor: 'instance creation' stamp: 'CWB 6/26/2010 21:15'!
fileIn: fullName
	"File in the entire contents of the file specified by the name provided"

	(self readOnlyFileNamed: fullName) fileIn; close.! !


!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 7/2/2010 20:32'!
fileMenu: aMenuTitleMorph

	| menu |
	menu _ CustomMenu new.
	menu add: 'New' action: #newScratchProject.
	menu add: 'Open' action: #openScratchProject.
	menu add: 'Save' action: #saveScratchProjectNoDialog.
	menu add: 'Save As' action: #saveScratchProject.
	menu addLine.
	menu add: 'Import Project' action: #importScratchProject.
	menu add: 'Export Sprite' action: #exportSprite.
	menu addLine.
	menu add: 'Project Notes' action: #editNotes.

	Sensor shiftPressed ifTrue: [  "developer menu"
		menu addLine.
		menu add: 'Write Project Summary' action: #writeSummaryFile.
		menu add: 'Write Multiple Project Summaries' action: #writeMultipleSummaries.
		menu addLine.
		fillScreenFlag
			ifTrue: [
				menu add: 'Exit User Mode' action: #fillScreenOff]
			ifFalse: [
				menu add: 'Enter User Mode' action: #fillScreenOn.
				menu add: 'Save Image in User Mode' action: #saveImageForEndUser].
		menu addLine.
		menu add: 'Import from version control' action: #versionControlImport.
		menu add: 'Export to version control' action: #versionControlExport.
		menu add: 'Apply latest UI skin' action: #updateInterfaceSkin ].

	menu addLine.
	menu add: 'Quit' action: #quitScratch.
	menu localize.
	
	#(2 4 5 6 7) do: [:n |
		menu labels at: n put:
			((menu labels at: n) copyFrom: 1 to: (menu labels at: n) size - 1), ScratchTranslator ellipsesSuffix].

	menu invokeOn: self at: aMenuTitleMorph bottomLeft + (0@10).

! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 10/8/2011 09:46'!
saveImageForEndUser

	self saveImageForEndUserWithOptions: #().! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 10/8/2011 10:52'!
saveImageForEndUserWithOptions: optionList
"Saves the image for the end user.  OptionList may be empty or include #alreadyConfirmed and #leaveSystemWindows"

	(optionList includes: #alreadyConfirmed) ifFalse: [
		(self confirm: '
Close non-Scratch windows and save this
image in end-user (fillScreen) mode?') ifFalse: [^ self]].


	ScratchFrameMorph isXO ifTrue: [Preferences useLargeFonts].

	self setLanguage: 'en'.

	(optionList includes: #leaveSystemWindows) ifFalse: [
		World submorphs do: [:m |
			(m isKindOf: SystemWindow) ifTrue: [m delete]]].
	self clearStage.
	ScratchFrameMorph ProcessedMetaCommands: false.

	Display newDepth: 32.
	self fillScreenOn.
	World doOneCycleNow.
	Smalltalk snapshot: true andQuit: true.
	self startup.
	Sensor useOSEvents: true.
! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 7/2/2010 21:30'!
updateInterfaceSkin

	"Reads in the latest skin from the ScratchSkin folder and updates the UI appearance."

	self class readSkinFrom: (FileDirectory default directoryNamed: 'ScratchSkin').
	"self rebuildUIForNewLanguage."
	ScratchTranslator updateScratchUI.! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 6/29/2010 13:20'!
versionControlExport
	"Outputs all classes that have changed since the project start date to a directory that should be managed by an external revision control system."

	| dir date |

	dir _ self class versionControlDirectory.
	date _ self class versionControlStartDate.
	
	dir fileOutClassesChangedSince: date.	
	! !

!ScratchFrameMorph methodsFor: 'menu/button actions' stamp: 'CWB 6/29/2010 13:21'!
versionControlImport
	"Updates the classes in the Squeak VM by importing them from a 'Classes' directory that should be managed by your revision control system."

	self class versionControlDirectory fileInContents

	
! !

!ScratchFrameMorph methodsFor: 'startup' stamp: 'CWB 10/8/2011 10:00'!
startup

	| startupFileNames fileName arg presentationMode |
	HostSystemMenus startUp.
	HostSystemMenus menuBarControler reviseHostMenus.
	ScriptableScratchMorph randomInit.
	ScratchTranslator detectRenderPlugin.
	ScratchTranslator importLanguagesList.

	self processSettingsFile.
	self readDefaultNotes.

	self updateProjectName.
	shuffledCostumeNames _ nil.
	author _ ''.
	loginName _ ''.
	loginPassword _ ''.
	justSaved _ true.
	presentationMode _ false.

	startupFileNames _ InputSensor startupFileNames asOrderedCollection.
	2 to: 10 do: [:i |
		arg _ Smalltalk getSystemAttribute: i.
		(arg notNil and: [arg size > 0]) ifTrue: [
			startupFileNames addLast: (ScratchPlugin primShortToLongPath: arg)]].

	startupFileNames do: [:n |
		(n asLowercase = 'presentation') ifTrue: [presentationMode _ true].
		(n asLowercase = 'fullscreen') ifTrue: [TakeOverScreen _ true]].

	TakeOverScreen ifTrue: [
		Smalltalk fullScreenMode: true.
		World restoreDisplay].

	self enterQuarterModeIfSmallScreen.

	fileName _ startupFileNames
		detect: [:fn |
			(fn asLowercase endsWith: '.sb') or: [fn asLowercase endsWith: '.scratch']]
		ifNone: [nil].
	fileName ifNotNil: [
		presentationMode ifTrue: [Display fillColor: Color black].
		self openScratchProjectNamed: fileName.
		presentationMode ifTrue: [self enterPresentationMode; shoutGo].
		^ self].

	viewerPane currentCategory: 'motion'.
	self setDefaultSprite.
	self newScratchProject.

	fileName _ startupFileNames
		detect: [:fn | fn asLowercase endsWith: '.sprite']
		ifNone: [ nil ].

	fileName ifNotNil: [
		"open a .sprite file"
		workPane submorphs do: [:m | (m isKindOf: ScratchSpriteMorph) ifTrue: [m deleteSprite]].
		self importSpriteOrProject: fileName.
	].

	"Look for commands to set the version, save the image, or import from version control"	
	ScratchFrameMorph processMetaCommandsFor: self.! !

!ScratchFrameMorph methodsFor: 'private' stamp: 'CWB 7/2/2010 21:32'!
rebuildUIForNewLanguage
	"Rebuild my UI after the language or font has been changed."

	World fullRepaintNeeded.
	viewerPane rebuildCategorySelectors.
	self updatePanes.
	logoMorph delete.
	self createLogo.
	self
		view: scriptsPane target
		tab: scriptsPane tabPane currentTab
		category: viewerPane currentCategory.

! !


!ScratchFrameMorph class methodsFor: 'utilities' stamp: 'CWB 10/10/2011 09:32'!
processMetaCommandsFor: frame
"Looks through the command line parameters for build-related commands."

	| arg args expectingVersion scratchFrame |

	ProcessedMetaCommands ifTrue: [ ^ self ].	"Avoid an infinite loop"
	ProcessedMetaCommands _ true.

	expectingVersion _ false.
	scratchFrame _ frame.

	args _ OrderedCollection new.
	2 to: 10 do: [:i |
		arg _ Smalltalk getSystemAttribute: i.
		arg ifNotNil: [args add: arg ]].

	args do: [:param |
		expectingVersion ifTrue: [ self version: param. expectingVersion _ false ].
		(param asLowercase = 'setversion') ifTrue: [ expectingVersion _ true. ].
		(param asLowercase = 'importsource') ifTrue: [ 
			frame fillScreenOff.
			frame updateInterfaceSkin.
			frame delete. 
			self versionControlDirectory fileInContents.
			ProcessedMetaCommands _ true.
			scratchFrame _ self recreateScratchFrame.
		].
		(param asLowercase = 'saveimageinusermode') ifTrue: [
			scratchFrame saveImageForEndUserWithOptions: #(#alreadyConfirmed).
		].
	].

! !

!ScratchFrameMorph class methodsFor: 'utilities' stamp: 'CWB 10/8/2011 11:04'!
recreateScratchFrame
"This is a specialized function designed to be sent after everything from version control
has been filed in (so that new functionality can be added here if necessary).  
The function is designed to recreate the scratch frame and return it at the end."

	| frame |

	frame _ self new.
	frame startup; newScratchProject; openInWorld.
	^ frame.! !

!ScratchFrameMorph class methodsFor: 'version control' stamp: 'CWB 6/29/2010 13:02'!
versionControlDirectory
	"Returns the directory used by the revision control directory to store filed-out classes."

	| baseDirectory subdirName |

	baseDirectory _ FileDirectory default.	"Use the folder the image is in"
	subdirName _ 'Classes'.

	"Create the source code revision directory is it doesn't exist"
	(baseDirectory directoryExists: subdirName) 
		ifFalse: [ baseDirectory createDirectory: subdirName ].

	"Return the code directory"
	^ baseDirectory directoryNamed: subdirName	! !

!ScratchFrameMorph class methodsFor: 'version control' stamp: 'CWB 6/29/2010 13:04'!
versionControlStartDate

	"Only classes that have changed since this date will be exported to the revision control system"

	^ '01/01/2010' asDate! !

!ScratchFrameMorph class methodsFor: 'accessing' stamp: 'CWB 10/7/2011 17:16'!
ProcessedMetaCommands
"Processed Meta Commands is used to avoid an infinite loop when rebuilding the ScratchFrameMorph from the command line."

	^ ProcessedMetaCommands.! !

!ScratchFrameMorph class methodsFor: 'accessing' stamp: 'CWB 10/7/2011 17:16'!
ProcessedMetaCommands: value
"Processed Meta Commands is used to avoid an infinite loop when rebuilding the ScratchFrameMorph from the command line."

	ProcessedMetaCommands _ value.! !


!Utilities class methodsFor: 'miscellaneous' stamp: 'CWB 6/28/2010 21:04'!
dateOfLatestChangeToClass: aClass

	"Returns the date of when a class last changed (or nil if it was long, long ago)"
 
	| dates |

	dates _ aClass selectors asOrderedCollection 
		collect: [ :sel | 
			self dateOfLatestChangeToMethod: (aClass compiledMethodAt: sel) ].
	dates _ dates reject: [ :date | date isNil ].
	dates _ dates asSortedCollection.
	(dates isEmpty) 
		ifFalse:	[^dates last]
		ifTrue: [^nil]
! !

!Utilities class methodsFor: 'miscellaneous' stamp: 'CWB 6/28/2010 21:48'!
dateOfLatestChangeToMethod: aMethod

"Returns the date (extracted from the timestamp) for when a method was last changed, or nil if it is very old"
| datestamp |
 
datestamp _ self timeStampForMethod: aMethod.

('* */*/* *' match: datestamp)	"initials date time"
"((datestamp size > 4) and: [(datestamp occurrencesOf: Character space) = 2])"
	ifFalse: [^nil] 
	ifTrue: [^
		((datestamp allButFirst: (datestamp findDelimiters: ' ' startingAt: 1)) withoutLeadingBlanks) asDate]
! !

Morph subclass: #ScratchFrameMorph
	instanceVariableNames: 'blockEditors topPane viewerPane scriptsPane stageFrame workPane titlePane libraryPane menuPanel stageButtonsPanel readoutPane logoMorph projectTitleMorph flagButton pauseButton fillScreenFlag paintingInProgress projectDirectory projectName projectInfo author loginName loginPassword watcherPositions shuffledCostumeNames justSaved viewModeButtons viewMode lastViewMode viewModeButtonsPanel toolbarPanel lastWeDoPoll enchantingDialog connectionInterfaces quitFlag '
	classVariableNames: 'AllowSharing Clipboard DefaultNotes DefaultSprite Fonts FontsXO IsXO ProcessedMetaCommands ScratchServers ScratchSkin ScratchSkinXO ShareServer ShareServerPath SupportServer SupportServerPath TakeOverScreen UseErrorCatcher Version VersionDate VisibleDrives WorkpaneExtent '
	poolDictionaries: ''
	category: 'Scratch-UI-Panes'!
